#include <fstream.h>
#include <iostream.h>
#include <string.h>
#include <stdlib.h>

// run with commmand - "bubble sortme.txt" or "bubble" and type "sortme.txt"
// or type "bubble rubbish.dat" or something to test error checking

/*
 *========================================================
 * This application is a bit different - the functions
 * come after the main program (i.e. where they are
 * from) so need to prototype the functions so the
 * compiler can do proper type checking. If the 
 * functions are written before the main program
 * then no need for the prototypes; bit like what
 * a header file does... try putting functions before main
 *========================================================
 */

// prototypes so the compiler doesn't puke... try commenting them out!
bool openFile(char*, ifstream&);
int readFile(ifstream&, float*, float*);
void bubbleSort(float*, int);
void writeOutput(float*, float*, int);

/*
 * entry point for application
 */
int main(int argc, char* argv[])
{
  // the input file stream object
  ifstream in;

  // if file name given on command line, send to function
  // NB: argv[0] is always the executable's name
  if (!openFile(argc > 1 ? argv[1] : 0, in)) {
    cerr << argv[0] << ": cannot open file\n";
    exit(-1);
  }

  // to simplify, assume data will be less than 100 numbers
  // will write dynamic array version of this on request
  float originalData[100], sortedData[100];
  int ndata = 0;

  // read in the data, if nothing read in then error
  if ((ndata = readFile(in, originalData, sortedData)) == 0) {
    cerr << argv[0] << ": error reading file\n";
    exit(-1);
  }
  else {
    in.close(); // we've finished with the file, close it (finite resources!)
  }

  // do the incredibly inefficient sort
  bubbleSort(sortedData, ndata);

  // write out the two arrays
  writeOutput(originalData, sortedData, ndata);

  // finish, telling the operating system it went well...
  return 0;
}

/*
 * ask for a file name, open the file, return success
 */
bool openFile(char* fromCommandLine, ifstream& in)
{
  char fileName[128]; // this assumes pathname is less than 128 characters; dodgy!

  if (fromCommandLine) {
    strcpy(fileName, fromCommandLine);
  }
  else {
    cout << "Enter file name: ";
    cin >> fileName;
  }

  in.open(fileName, ios::in|ios::nocreate);

  return (in.is_open() != 0);
}

/*
 * read the file, fill the float arrays
 */
int readFile(ifstream& in, float* data1, float* data2)
{
  float temp;
  int n = 0;

  while (!in.eof()) {
    in >> temp;
    if (!(in.rdstate() & ios::failbit)) { // is input sensible?
      data1[n] = data2[n] = temp;
      n++;
    }
  }

  return n;
}

/*
 * bubble sort the array (ascending order)
 */
void bubbleSort(float* data, int n)
{
  for (int i=0; i<n; i++) {
    bool finished = true;
    for (int j=1; j<n; j++) {
      if (data[j-1] > data[j]) {
        float temp = data[j-1];
        data[j-1] = data[j];
        data[j] = temp;
        finished = false;
      }
    }
    if (finished) break; // got thru the array without swapping; out of here
  }
}

/*
 * write original and sorted data to the console
 */
void writeOutput(float* data1, float* data2, int n)
{
  cout << "\nResults: Original\tSorted\n"; // \t is a tab, \n is a new line

  for (int i=0; i<n; i++) {
    cout << "\t" << data1[i] << "\t\t" << data2[i] << "\n";
  }
}
