#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>

/*
 * this program copies one file to another...
 * on the command line you specify the from_file and the to_file
 * e.g. M:\C++> copyfile rubbish.txt crap.txt
 * this will copy the contents of 'rubbish' into a new file called 'crap'
 * This works OK for simple text files - try it with a binary file!
 */

// prototypes... 
bool openInput(const char*, ifstream&);
bool openOutput(const char*, ofstream&);
void copyFromTo(ifstream&, ofstream&);

/*
 * entry point
 */
int main(int argc, char* argv[])
{
  if (argc < 3) {
    cerr << "usage: " << argv[0] << " <from_file> <to_file>\n";
    exit(-1);
  }

  ifstream in;
  if (!openInput(argv[1], in)) {
    cerr << argv[0] << ": cannot open input file\n";
    exit(-1);
  }

  ofstream out;
  if (!openOutput(argv[2], out)) {
    cerr << argv[0] << ": cannot open output file\n";
    exit(-1);
  }

  copyFromTo(in, out);

  in.close();
  out.close();

  cout << "copy successful\n";

  return 0;
}

/*
 * open the input file
 */
bool openInput(const char* fileName, ifstream& inFile)
{
  inFile.open(fileName, ios::in|ios::nocreate);

  return (inFile.is_open() != 0);
}

/*
 * open the output file
 */
bool openOutput(const char* fileName, ofstream& outFile)
{
  outFile.open(fileName, ios::out);

  return (outFile.is_open() != 0);
}

/*
 * copy the contents of From into To
 */
void copyFromTo(ifstream& from, ofstream& to)
{
  char buffer[1024];

  while (!from.eof()) {
    from.getline(buffer, 1024); // read the next line of the file
    if (!(from.rdstate() & ios::failbit)) { // check read was OK
      to << buffer << "\n"; // write the line to file
    }
  }
}
